module C4
 
  use S
  use w

  implicit none
  integer,private                           	:: nz,nx,z_pad,x_pad,ntw
  real,private                              	:: dz,dx,dt
  real,dimension(:,:),allocatable,private   	:: vel
  real,dimension(:,:,:),allocatable,private		:: temp1,temp2,temp3

  contains

  subroutine C4_init(nz_in,nx_in,dt_in,vel_in,dz_in,dx_in,z_pad_in,x_pad_in,ntw_in)
    integer                :: nz_in,nx_in,z_pad_in,x_pad_in,ntw_in
    real                   :: dt_in,dz_in,dx_in
    real,dimension(:,:)    :: vel_in

    nz = nz_in
    nx = nx_in
    z_pad = z_pad_in
    x_pad = x_pad_in
    ntw = ntw_in
    dz = dz_in
    dx = dx_in
    dt = dt_in

    if (.not. allocated(vel)) allocate(vel(nz,nx))
    vel = vel_in

    call S_init(nz,nx,ntw,dt,vel)
    call w_init(nz,nx,dt,vel,dz,dx,z_pad,x_pad)
    
  	if (.not. allocated(temp1)) allocate(temp1(nz,nx,ntw))
   	if (.not. allocated(temp2)) allocate(temp2(nz,nx,ntw))
   	if (.not. allocated(temp3)) allocate(temp3(nz,nx,ntw))      

  end subroutine

  function C4_op(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call C4_op2(adj,add,model,data)   
    stat=0
  end function

  subroutine C4_op2(adj,add,model,data)
    logical,intent(in)                      :: adj,add 
    real,dimension(nz,nx,ntw)               :: model,data
    integer                                 :: it

    if(adj) then 
        if (.not. add) model=0.
        do it=1,ntw
            call w_op2(adj,.false.,temp1(:,:,it),data(:,:,it))
            temp2(:,:,it) = data(:,:,it) - temp1(:,:,it)
        end do
        call S_op2(adj,.false.,temp3,temp2)
        model = model + temp3
    else 
        if(.not. add) data = 0.
        call S_op2(adj,.false.,model,temp1)
        do it=1,ntw
            call w_op2(adj,.false.,temp1(:,:,it),temp2(:,:,it))
            temp3(:,:,it) = temp1(:,:,it) - temp2(:,:,it)
        end do
        data = data + temp3
    end if
        
  end subroutine

  subroutine C4_close()
    deallocate(vel)
  end subroutine

end module
