module C5

	use C2
	use weight
	use J

	implicit none

	integer,private                           :: nz,nx,z_pad,x_pad
	real,private                              :: dz,dx,dt
	real,dimension(:,:),allocatable,private   :: vel,temp1,temp2

	contains

	subroutine C5_init(nz_in,nx_in,dt_in,vel_in,dz_in,dx_in,z_pad_in,x_pad_in)
		integer                :: nz_in,nx_in,z_pad_in,x_pad_in
		real                   :: dt_in,dz_in,dx_in
		real,dimension(:,:)    :: vel_in

		nz = nz_in
		nx = nx_in
		dz = dz_in
		dx = dx_in
		dt = dt_in
		z_pad = z_pad_in
		x_pad = x_pad_in

		if (.not. allocated(vel)) allocate(vel(nz,nx))
		vel = vel_in

		if (.not. allocated(temp1)) allocate(temp1(nz,nx))
		if (.not. allocated(temp2)) allocate(temp2(nz,nx))		

		call weight_init(nz,nx,dt,vel,dz,dx,z_pad,x_pad)
		call C2_init(nz,nx,dt,vel,dz,dx)
		call J_init(nz,nx,z_pad,x_pad,dz,dx)

	end subroutine

	function C5_op(adj,add,model,data) result(stat)
		logical,intent(in)     :: adj,add
		real,dimension(:)      :: model,data
		integer                :: stat
		call C5_op2(adj,add,model,data)   
		stat=0
	end function

	subroutine C5_op2(adj,add,model,data)
		logical,intent(in)                :: adj,add 
		real,dimension(nz,nx)             :: model,data
		real,dimension(nz,nx)             :: temp1,temp2

		if (adj) then 
			if (.not. add) model=0.

			call weight_op2(adj,.false.,temp1,data)
			model = model + temp1
			temp1 = vel * dt * temp1
			call J_op2(adj,.false.,temp2,temp1)
			model = model - temp2

			call weight_op2(adj,.false.,temp1,data)
			temp1 = data - temp1
			call C2_op2(adj,.false.,temp2,temp1)
			model = model + temp2

		else 
		
			if(.not. add) data = 0.

			call C2_op2(.false.,.false.,model,temp1)
			call weight_op2(.false.,.false.,temp1,temp2)
			data = data + temp1 - temp2

			call J_op2(.false.,.false.,model,temp1)
			temp1 = vel * dt * temp1 
			temp1 = model - temp1
			call weight_op2(.false.,.true.,temp1,data)

		end if
	
	end subroutine

end module
