! computes the second order derivative in x-direction with 10th order precision
module Dzz

  implicit none
  integer,private                   :: nz,nx,buffer
  real,private                      :: dz
  real,private                      :: C0z,C1z,C2z,C3z,C4z,C5z
  real,dimension(:,:),allocatable   :: temp

  contains

  subroutine Dzz_init(nz_in,nx_in,dz_in)
    integer        :: nz_in,nx_in
    real           :: dz_in
    
    nz = nz_in
    nx = nx_in
    dz = dz_in

    buffer = 5

    if (.not. allocated(temp)) allocate(temp(1-buffer:nz+buffer,nx))
    temp = 0.0

    C0z = -2.927222222/(dz*dz)
    C1z =  1.666666667/(dz*dz)
    C2z = -0.238095238/(dz*dz)
    C3z =  0.039682539/(dz*dz)
    C4z = -0.004960317/(dz*dz)
    C5z =  0.000317460/(dz*dz)

  end subroutine

  function Dzz_op(adj,add,model,data) result(stat)
    logical,intent(in)   :: adj,add
    real,dimension(:)  :: model,data
    integer              :: stat
    call Dzz_op2(adj,add,model,data)
    stat=0
  end function

  subroutine Dzz_op2(adj,add,model,data)
    logical,intent(in)       :: adj,add
    real,dimension(nz,nx)  :: model,data
    integer                  :: iz,ix

    if(adj) then
        if (.not. add) model=0.
        temp(1:nz,1:nx) = data

        !$OMP PARALLEL DO private(iz,ix)
        do ix=1,nx
            do iz=1,nz
                model(iz,ix) = model(iz,ix) + &
                                      C0z *  temp(iz,ix) + &
                                      C1z * (temp(iz+1,ix) + temp(iz-1,ix)) + &
                                      C2z * (temp(iz+2,ix) + temp(iz-2,ix)) + &
                                      C3z * (temp(iz+3,ix) + temp(iz-3,ix)) + &
                                      C4z * (temp(iz+4,ix) + temp(iz-4,ix)) + &
                                      C5z * (temp(iz+5,ix) + temp(iz-5,ix))

                end do
            end do
        !$OMP END PARALLEL DO

    else
        if(.not. add) data = 0.
        temp(1:nz,1:nx) = model

        !$OMP PARALLEL DO private(iz,ix)
        do ix=1,nx
            do iz=1,nz
                data(iz,ix) = data(iz,ix) + &
                                      C0z *  temp(iz,ix) + &
                                      C1z * (temp(iz+1,ix) + temp(iz-1,ix)) + &
                                      C2z * (temp(iz+2,ix) + temp(iz-2,ix)) + &
                                      C3z * (temp(iz+3,ix) + temp(iz-3,ix)) + &
                                      C4z * (temp(iz+4,ix) + temp(iz-4,ix)) + &
                                      C5z * (temp(iz+5,ix) + temp(iz-5,ix))
            end do
        end do
        !$OMP END PARALLEL DO

    end if

  end subroutine

  subroutine Dzz_close()
    deallocate(temp)
  end subroutine

end module
