module Kr

	implicit none
  	integer,private                                    :: nx,nz,ntr,nb_rec,shift
  	integer,dimension(:,:),allocatable,private         :: rec_coord
  	logical,private									   :: dipole

  	contains

  	subroutine Kr_init(nz_in,nx_in,ntr_in,rec_coord_in,dipole_in,shift_in)
    	integer                     :: nx_in,nz_in,ntr_in,shift_in
    	integer,dimension(:,:)      :: rec_coord_in
    	logical						:: dipole_in

    	nx  = nx_in
    	nz  = nz_in
    	ntr = ntr_in
    	nb_rec = size(rec_coord_in,1)
    	dipole = dipole_in
    	shift = 0
    	if (dipole .eqv. .true.) shift = shift_in
    	if (.not. allocated(rec_coord)) allocate(rec_coord(nb_rec,2))
    	rec_coord = rec_coord_in

  	end subroutine

  	function Kr_op(adj,add,model,data) result(stat)
    	logical,intent(in)   :: adj, add
    	real,dimension(:)    :: model,data
    	integer              :: stat
    	call Kr_op2(adj,add,model,data)
    	stat=0
  	end function

  	subroutine Kr_op2(adj,add,model,data)
    	logical                          :: adj,add
    	real,dimension(nz,nx,ntr)        :: data
    	real,dimension(nb_rec,ntr)       :: model
    	integer                          :: ir

    	if (dipole .eqv. .true.) then

    		if (adj) then
        		if(add .eqv. .false.) model = 0.
			
        		!$OMP PARALLEL DO private(ir)
        		do ir=1,nb_rec
            		model(ir,:) = model(ir,:) + data(rec_coord(ir,1),rec_coord(ir,2),:)
                	model(ir,:) = model(ir,:) - data(rec_coord(ir,1)+shift,rec_coord(ir,2),:)
        		end do
        		!$OMP END PARALLEL DO

    		else
        		if(add .eqv. .false.) data = 0.

        		!$OMP PARALLEL DO private(ir)
        		do ir=1,nb_rec
                	data(rec_coord(ir,1),rec_coord(ir,2),:) = data(rec_coord(ir,1),rec_coord(ir,2),:) + model(ir,:)
                	data(rec_coord(ir,1)+shift,rec_coord(ir,2),:) = data(rec_coord(ir,1)+shift,rec_coord(ir,2),:) - model(ir,:)
        		end do
        		!$OMP END PARALLEL DO

    		end if

    	else
    		if (adj) then
        		if(add .eqv. .false.) model = 0.

        		!$OMP PARALLEL DO private(ir)
        		do ir=1,nb_rec
                	model(ir,:) = model(ir,:) + data(rec_coord(ir,1),rec_coord(ir,2),:)
        		end do
	        	!$OMP END PARALLEL DO

    		else
        		if(add .eqv. .false.) data = 0.

        		!$OMP PARALLEL DO private(ir)
        		do ir=1,nb_rec
                	data(rec_coord(ir,1),rec_coord(ir,2),:) = data(rec_coord(ir,1),rec_coord(ir,2),:) + model(ir,:)
        		end do
 		        !$OMP END PARALLEL DO

    		end if
		end if

  	end subroutine

end module
