module Ks

	implicit none
  	integer,private                                    :: nx,nz,nts,nb_src,shift
  	integer,dimension(:,:),allocatable,private         :: src_coord
  	logical,private									   :: dipole

  	contains

  	subroutine Ks_init(nz_in,nx_in,nts_in,src_coord_in,dipole_in,shift_in)
    	integer                     			:: nx_in,nz_in,nts_in,shift_in
    	integer,dimension(:,:)                  :: src_coord_in
    	logical									:: dipole_in

    	nx  = nx_in
    	nz  = nz_in
    	nts = nts_in
    	dipole = dipole_in
    	nb_src = size(src_coord_in,1)
    	shift = 0
    	if (dipole .eqv. .true.) shift = shift_in
    	if (.not. allocated(src_coord)) allocate(src_coord(nb_src,2))
    	src_coord = src_coord_in

  	end subroutine

  	function Ks_op(adj,add,model,data) result(stat)
    	logical,intent(in)   :: adj, add
    	real,dimension(:)    :: model,data
    	integer              :: stat
    	call Ks_op2(adj,add,model,data)
    	stat=0
  	end function

  	subroutine Ks_op2(adj,add,model,data)
    	logical                          :: adj,add
    	real,dimension(nz,nx,nts)        :: data
    	real,dimension(nb_src,nts)       :: model
    	integer                          :: is

    	if (dipole .eqv. .true.) then

    		if (adj) then
        		if(add .eqv. .false.) model = 0.
			
        		!$OMP PARALLEL DO private(is)
        		do is=1,nb_src
            		model(is,:) = model(is,:) + data(src_coord(is,1),src_coord(is,2),:)
                	model(is,:) = model(is,:) - data(src_coord(is,1)+shift,src_coord(is,2),:)
        		end do
        		!$OMP END PARALLEL DO

    		else
        		if(add .eqv. .false.) data = 0.

        		!$OMP PARALLEL DO private(is)
        		do is=1,nb_src
                	data(src_coord(is,1),src_coord(is,2),:) = data(src_coord(is,1),src_coord(is,2),:) + model(is,:)
                	data(src_coord(is,1)+shift,src_coord(is,2),:) = data(src_coord(is,1)+shift,src_coord(is,2),:) - model(is,:)				
        		end do
        		!$OMP END PARALLEL DO

    		end if

    	else
    		if (adj) then
        		if(add .eqv. .false.) model = 0.

        		!$OMP PARALLEL DO private(is)
        		do is=1,nb_src
                	model(is,:) = model(is,:) + data(src_coord(is,1),src_coord(is,2),:)
        		end do
	        	!$OMP END PARALLEL DO

    		else
        		if(add .eqv. .false.) data = 0.

        		!$OMP PARALLEL DO private(is)
        		do is=1,nb_src
                	data(src_coord(is,1),src_coord(is,2),:) = data(src_coord(is,1),src_coord(is,2),:) + model(is,:)
        		end do
 		        !$OMP END PARALLEL DO

    		end if
		end if

  	end subroutine

end module
