module Lr

  implicit none
  integer,private        :: nx,nz,ntr,ntw
  real,private           :: dtr,dtw,otr,otw,scale,oversamp

  contains

  subroutine Lr_init(nz_in,nx_in,ntr_in,ntw_in,otr_in,dtr_in,otw_in,dtw_in)
    integer       :: nx_in,nz_in,ntr_in,ntw_in
    real          :: otr_in,dtr_in,otw_in,dtw_in

    nx  = nx_in
    nz  = nz_in
    ntr = ntr_in
    ntw = ntw_in
    dtr = dtr_in
    otr = otr_in
    dtw = dtw_in
    otw = otw_in
    oversamp = (1.0 * ntw)/(1.0 * ntr)
    scale = 1.0 / sqrt(oversamp)

  end subroutine

  function Lr_op(adj,add,model,data) result(stat)
    logical,intent(in)   :: adj, add
    real,dimension(:)    :: model,data
    integer              :: stat
    call Lr_op2(adj,add,model,data)
    stat=0
  end function

  subroutine Lr_op2(adj,add,model,data)
    logical                          :: adj,add
    real,dimension(nz,nx,ntw)        :: data
    real,dimension(nz,nx,ntr)        :: model
    integer                          :: iz,ix,it,mult_int,index_inf,index_sup
    real                             :: coord,mult_float,fraction

    if (adj) then 
        if(add .eqv. .false.) model = 0.
        do it=1,ntw
            coord = otw + (it-1) * dtw
            mult_float = (coord - otr) / dtr
            mult_int = mult_float
            fraction = mult_float - mult_int * 1.0
            index_inf = mult_int + 1
            index_sup = index_inf + 1

            if ((index_inf .ge. 1) .and. (index_sup .le. ntr)) then
                !$OMP PARALLEL DO private(iz,ix)
                do ix=1,nx
                    do iz=1,nz
                        model(iz,ix,index_inf) = model(iz,ix,index_inf) + data(iz,ix,it) *(1.0 - fraction) * scale
                        model(iz,ix,index_sup) = model(iz,ix,index_sup) + data(iz,ix,it) * fraction * scale
                    end do
                end do
                !$OMP END PARALLEL DO
            end if
        end do

    else
        if(add .eqv. .false.) data = 0
        do it=1,ntw
            coord = otw + (it-1) * dtw
            mult_float = (coord - otr) / dtr
            mult_int = mult_float
            fraction = mult_float - mult_int * 1.0
            index_inf = mult_int + 1
            index_sup = index_inf + 1

            if ((index_inf .ge. 1) .and. (index_sup .le. ntr)) then
                !$OMP PARALLEL DO private(iz,ix)
                do ix=1,nx
                    do iz=1,nz
                        data(iz,ix,it) = data(iz,ix,it) + model(iz,ix,index_inf) * (1.0 - fraction) * scale + model(iz,ix,index_sup) * fraction * scale
                    end do
                end do
            	!$OMP END PARALLEL DO
            end if

        end do

    end if

  end subroutine 

end module
