module Lr_2d

  implicit none
  integer,private          :: nx,nz,ntr,ntw,nr
  real,private             :: dtr,dtw,otr,otw,scale,oversamp

  contains

  subroutine Lr_2d_init(ntr_in,ntw_in,otr_in,dtr_in,otw_in,dtw_in,nr_in)
    integer         :: ntr_in,ntw_in,nr_in
    real            :: otr_in,dtr_in,otw_in,dtw_in

    ntr = ntr_in
    ntw = ntw_in
    dtr = dtr_in
    otr = otr_in
    dtw = dtw_in
    otw = otw_in
    nr  = nr_in
    oversamp = (1.0 * ntw)/(1.0 * ntr)
    scale = 1.0 / sqrt(oversamp)

  end subroutine

  function Lr_2d_op(adj,add,model,data) result(stat)
    logical,intent(in)     :: adj, add
    real,dimension(:)    :: model,data
    integer                :: stat
    call Lr_2d_op2(adj,add,model,data)
    stat=0
  end function

  subroutine Lr_2d_op2(adj,add,model,data)
    logical                            :: adj,add
    real,dimension(ntw,nr)             :: data
    real,dimension(ntr,nr)             :: model
    integer                            :: it,mult_int,index_inf,index_sup,ir
    real                               :: coord,mult_float,fraction

    if (adj) then 
        if(add .eqv. .false.) model = 0.
			
        !$OMP PARALLEL DO private(ir,it,coord,mult_float,mult_int,fraction,index_inf,index_sup)	
		do ir=1,nr
		    do it=1,ntw
            	coord = otw + (it-1) * dtw
                mult_float = (coord - otr) / dtr
                mult_int   = mult_float
                fraction = mult_float - mult_int * 1.0
                index_inf = mult_int + 1
                index_sup = index_inf + 1
                if ((index_inf .ge. 1) .and. (index_sup .le. ntr)) then
                    model(index_inf,ir) = model(index_inf,ir) + data(it,ir) *(1.0-fraction) * scale
                    model(index_sup,ir) = model(index_sup,ir) + data(it,ir) * fraction * scale
                end if
            end do
         end do
        !$OMP END PARALLEL DO
    else
        if(add .eqv. .false.) data = 0.
        !$OMP PARALLEL DO private(ir,it,coord,mult_float,mult_int,fraction,index_inf,index_sup)	
		do ir=1,nr
            do it=1,ntw
                coord = otw + (it-1) * dtw
                mult_float = (coord - otr) / dtr
                mult_int   = mult_float
                fraction = mult_float - mult_int * 1.0
                index_inf = mult_int + 1
                index_sup = index_inf + 1
                if ((index_inf .ge. 1) .and. (index_sup .le. ntr)) then
                	data(it,ir) = data(it,ir) + model(index_inf,ir) * (1.0-fraction) * scale + model(index_sup,ir) * fraction * scale
                end if
            end do
		end do 
        !$OMP END PARALLEL DO		       


    end if

  end subroutine 

end module
