module Ls_1d

  implicit none
  integer,private      :: nts,ntw
  real,private         :: dts,dtw,ots,otw,scale,oversamp

  contains

  subroutine Ls_1d_init(nts_in,ntw_in,ots_in,dts_in,otw_in,dtw_in)
    integer       :: nts_in,ntw_in
    real          :: ots_in,dts_in,otw_in,dtw_in

    nts = nts_in
    ntw = ntw_in
    dts = dts_in
    ots = ots_in
    dtw = dtw_in
    otw = otw_in
    oversamp = (1.0 * ntw)/(1.0 * nts)
    scale = 1.0 / sqrt(1.0 * oversamp)

  end subroutine

  function Ls_1d_op(adj,add,model,data) result(stat)
    logical,intent(in)   :: adj, add
    real,dimension(:)    :: model,data
    integer              :: stat
    call Ls_1d_op2(adj,add,model,data)
    stat=0
  end function

  subroutine Ls_1d_op2(adj,add,model,data)
    logical	                   :: adj,add
    real,dimension(ntw)        :: data
    real,dimension(nts)        :: model
    integer                    :: it,mult_int,index_inf,index_sup
    real                       :: coord,mult_float,fraction

    if (adj) then 
        if(add .eqv. .false.) model = 0.
        do it=1,ntw
            coord = otw + (it-1) * dtw
            mult_float = (coord - ots) / dts
            mult_int = mult_float
            fraction = mult_float - mult_int * 1.0
            index_inf = mult_int + 1
            index_sup = index_inf + 1
            if ((index_inf .ge. 1) .and. (index_sup .le. nts)) then
	            model(index_inf) = model(index_inf) + data(it) *(1.0 - fraction) * scale
    	        model(index_sup) = model(index_sup) + data(it) * fraction * scale

            end if
        end do

    else
        if(add .eqv. .false.) data = 0
        do it=1,ntw
            coord = otw + (it-1) * dtw
            mult_float = (coord - ots) / dts
            mult_int = mult_float
            fraction = mult_float - mult_int * 1.0
            index_inf = mult_int + 1
            index_sup = index_inf + 1
            if ((index_inf .ge. 1) .and. (index_sup .le. nts)) then
	            data(it) = data(it) + model(index_inf) * (1.0 - fraction) * scale + model(index_sup) * fraction * scale
            end if

        end do

    end if

  end subroutine 

end module
