module S

  implicit none
  integer,private                            :: nz,nx,ntw
  real,dimension(:,:),allocatable,private    :: coeff
  real,private                               :: dt

  contains

  subroutine S_init(nz_in,nx_in,ntw_in,dt_in,vel_in)  
    integer                     :: nz_in,nx_in,ntw_in
    real,dimension(:,:)         :: vel_in
    real                        :: dt_in

    nz = nz_in
    nx = nx_in
    ntw = ntw_in
    dt = dt_in

    if (.not. allocated(coeff)) allocate(coeff(nz,nx))
    coeff = vel_in * vel_in * dt * dt

  end subroutine

  function S_op(adj,add,model,data) result(stat)
    logical,intent(in)   :: adj,add
    real,dimension(:)    :: model,data
    integer              :: stat

    call S_op2(adj,add,model,data)
    stat=0
  end function

  subroutine S_op2(adj,add,model,data)
    logical                          :: adj,add
    real,dimension(nz,nx,ntw)        :: model,data
    integer                          :: it,iz,ix

    if (adj) then 

        if(add .eqv. .false.) model = 0.
        do it=1,ntw
            !$OMP PARALLEL DO private(iz,ix)
            do ix=1,nx
                do iz=1,nz
                    model(iz,ix,it) = model(iz,ix,it) + data(iz,ix,it) * coeff(iz,ix)
                end do
            end do
            !$OMP END PARALLEL DO
        end do
    else

        if(add .eqv. .false.) data = 0.
        do it=1,ntw
            !$OMP PARALLEL DO private(iz,ix)
            do ix=1,nx
                do iz=1,nz
                    data(iz,ix,it) = data(iz,ix,it) + model(iz,ix,it) * coeff(iz,ix)
                end do
            end do
            !$OMP END PARALLEL DO
        end do
    end if

  end subroutine 

  subroutine S_close()  
    deallocate(coeff)
  end subroutine

end module
