module dso

	implicit none

	integer,private                           :: nz,nx,nhz,nhx,nht
	integer,private							  :: hzmax,hxmax,htmax

	contains

	subroutine dso_init(nz_in,nx_in,hzmax_in,hxmax_in,htmax_in)
		integer                  :: nz_in,nx_in
		integer					 :: hzmax_in,hxmax_in,htmax_in

		nz = nz_in
		nx = nx_in
		hzmax = hzmax_in
		hxmax = hxmax_in
		htmax = htmax_in
		
	end subroutine

	function dso_op(adj,add,model,data) result(stat)
		logical,intent(in)       :: adj,add
		real,dimension(:)        :: model,data
		integer                  :: stat
		call dso_op2(adj,add,model,data)
		stat=0
	end function

	subroutine dso_op2(adj,add,model,data)
		logical,intent(in)                       						   :: adj,add
		real,dimension(nz,nx,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax)       :: model,data
		integer			        		         						   :: iz,ix,ihz,ihx,iht
		real															   :: weight

		if(adj) then
			if (.not. add) model=0.

			!$OMP PARALLEL DO private(iht,ihx,ihz,ix,iz,weight)
			do iht = -htmax,htmax
				do ihx = -hxmax,hxmax
					do ihz = -hzmax,hzmax				
						weight = sqrt(float(ihz*ihz + ihx*ihx + iht*iht))
						do ix = 1,nx
							do iz = 1,nz
								model(iz,ix,ihz,ihx,iht) = model(iz,ix,ihz,ihx,iht) + weight * data(iz,ix,ihz,ihx,iht)
							end do
						end do
					end do		
				end do
			end do
			!$OMP END PARALLEL DO
		else
			if(.not. add) data = 0.

			!$OMP PARALLEL DO private(iht,ihx,ihz,ix,iz,weight)
			do iht = -htmax,htmax
				do ihx = -hxmax,hxmax
					do ihz = -hzmax,hzmax	
						weight = sqrt(float(ihz*ihz + ihx*ihx + iht*iht))			
						do ix = 1,nx
							do iz = 1,nz
								data(iz,ix,ihz,ihx,iht) = data(iz,ix,ihz,ihx,iht) + weight * model(iz,ix,ihz,ihx,iht)
							end do
						end do
					end do		
				end do
			end do	
			!$OMP END PARALLEL DO
		end if

	end subroutine

end module
