program dso_main

	use sep
	use dso

  	implicit none

  	integer              					  :: nz2,nx2,n3
  	integer									  :: hzmax,hxmax,htmax,nhz,nhx,nht
  	real									  :: dot1,dot2,error
  	real									  :: dz,dx,dhz,dhx,dht
  	real									  :: oz,ox,ohz,ohx,oht  	
	real,dimension(:,:,:,:,:),allocatable     :: reflectivity_in,reflectivity_out
	real,dimension(:,:),allocatable     	  :: background_in,background_out
	logical									  :: adj
	integer*8								  :: sreed,rc,srite,wc
	integer*8								  :: nbytes	

  	call sep_init()
  	
	call from_history("n1",nz2)
	call from_history("d1",dz)	
	call from_history("o1",oz)	

	call from_history("n2",nx2)
	call from_history("d2",dx)
	call from_history("o2",ox)

	call from_param("hzmax",hzmax)
	call from_param("hxmax",hxmax)
	call from_param("htmax",htmax)	

	call from_history("n3",n3)
	call from_param("adj",adj)
	
	nhz = 1 + 2 * hzmax
	nhx = 1 + 2 * hxmax
	nht = 1 + 2 * htmax		
	
	allocate(background_in(nz2,nx2))
	allocate(background_out(nz2,nx2))
	allocate(reflectivity_in(nz2,nx2,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax))
	allocate(reflectivity_out(nz2,nx2,-hzmax:hzmax,-hxmax:hxmax,-htmax:htmax))	
	
	! initialize operator
	call dso_init(nz2,nx2,hzmax,hxmax,htmax)
	
	! read the background 
	nbytes = 4
	nbytes = nbytes * nx2 * nz2
	rc = sreed("in",background_in,nbytes)
	if(rc .eq. nbytes) then
		write(0,*) "read background [OK]"	
	else
		write(0,*) "error reading background"
		stop
	end if

	! set background_out to zero and write to output
	background_out = 0.0
	nbytes = 4
	nbytes = nbytes * nx2 * nz2 
  	wc = srite("out",background_out,nbytes)
	
	if(rc .eq. nbytes) then
		write(0,*) "read background [OK]"	
	else
		write(0,*) "error reading background"
		stop
	end if	

	! read the reflectivity
	nbytes = 4
	nbytes = nbytes * nx2 * nz2 * nhz * nhx * nht
	rc = sreed("in",reflectivity_in,nbytes)
	if(rc .eq. nbytes) then
		write(0,*) "read reflectivity [OK]"	
	else
		write(0,*) "error reading reflectivity"
		stop
	end if

	! apply DSO operator
	if (.not. adj) then

		! apply DSO forward to reflectivity_in
		call dso_op2(.false.,.false.,reflectivity_in,reflectivity_out) 
		nbytes = 4
		nbytes = nbytes * nx2 * nz2 * nhz * nhx * nht
	   	wc = srite("out",reflectivity_out,nbytes)
		
	else

		! apply DSO adjoint to reflectivity_in
		call dso_op2(.true.,.false.,reflectivity_out,reflectivity_in) 
		nbytes = 4
		nbytes = nbytes * nx2 * nz2 * nhz * nhx * nht
	   	wc = srite("out",reflectivity_out,nbytes)	
		
	end if


	call to_history("n1",nz2)
	call to_history("d1",dz)	
	call to_history("o1",oz)	

	call to_history("n2",nx2)
	call to_history("d2",dx)
	call to_history("o2",ox)

	call to_history("n3",n3)

	call sep_close()					
	
end program