module laplace10

  implicit none
  integer,private                   		:: nz,nx,buffer
  real,private                      		:: dx,dz
  real,private                      		:: C0z,C1z,C2z,C3z,C4z,C5z,C0x,C1x,C2x,C3x,C4x,C5x
  real,dimension(:,:),allocatable,private   :: temp

  contains

  subroutine laplace10_init(nz_in,nx_in,dz_in,dx_in)
    integer            :: nz_in,nx_in
    real               :: dz_in,dx_in
    
    nz = nz_in
    nx = nx_in
    dx = dx_in
    dz = dz_in

    buffer = 5

    if (.not. allocated(temp)) allocate(temp(1-buffer:nz+buffer,1-buffer:nx+buffer))
    temp = 0.0

    C0z = -2.927222222/(dz*dz)
    C1z =  1.666666667/(dz*dz)
    C2z = -0.238095238/(dz*dz)
    C3z =  0.039682539/(dz*dz)
    C4z = -0.004960317/(dz*dz)
    C5z =  0.000317460/(dz*dz)

    C0x = -2.927222222/(dx*dx)
    C1x =  1.666666667/(dx*dx)
    C2x = -0.238095238/(dx*dx)
    C3x =  0.039682539/(dx*dx)
    C4x = -0.004960317/(dx*dx)
    C5x =  0.000317460/(dx*dx)

  end subroutine

  function laplace10_op(adj,add,model,data) result(stat)
    logical,intent(in) :: adj,add
    real,dimension(:)  :: model,data
    integer            :: stat
    call laplace10_op2(adj,add,model,data)
    stat=0
  end function

  subroutine laplace10_op2(adj,add,model,data)
    logical,intent(in)     :: adj,add
    real,dimension(nz,nx)  :: model,data
    integer                :: iz,ix

    if(adj) then
        if (.not. add) model=0.
        temp(1:nz,1:nx) = data

        !$OMP PARALLEL DO private(iz,ix)
        do ix=1,nx
            do iz=1,nz
                model(iz,ix) = model(iz,ix) + C0z *temp(iz,ix) + &
                                      C1z * (temp(iz+1,ix) + temp(iz-1,ix)) + &
                                      C2z * (temp(iz+2,ix) + temp(iz-2,ix)) + &
                                      C3z * (temp(iz+3,ix) + temp(iz-3,ix)) + &
                                      C4z * (temp(iz+4,ix) + temp(iz-4,ix)) + &
                                      C5z * (temp(iz+5,ix) + temp(iz-5,ix)) + &
                                      C0x *  temp(iz,ix) + &
                                      C1x * (temp(iz,ix+1) + temp(iz,ix-1)) + &
                                      C2x * (temp(iz,ix+2) + temp(iz,ix-2)) + &
                                      C3x * (temp(iz,ix+3) + temp(iz,ix-3)) + &
                                      C4x * (temp(iz,ix+4) + temp(iz,ix-4)) + &
                                      C5x * (temp(iz,ix+5) + temp(iz,ix-5))

                end do
            end do
        !$OMP END PARALLEL DO

    else
        if(.not. add) data = 0.
        temp(1:nz,1:nx) = model

        !$OMP PARALLEL DO private(iz,ix)

        do ix=1,nx
            do iz=1,nz

                data(iz,ix) = data(iz,ix) + C0z *  temp(iz,ix) + &
                                    C1z * (temp(iz+1,ix) + temp(iz-1,ix)) + &
                                    C2z * (temp(iz+2,ix) + temp(iz-2,ix)) + &
                                    C3z * (temp(iz+3,ix) + temp(iz-3,ix)) + &
                                    C4z * (temp(iz+4,ix) + temp(iz-4,ix)) + &
                                    C5z * (temp(iz+5,ix) + temp(iz-5,ix)) + &
                                    C0x *  temp(iz,ix) + &
                                    C1x * (temp(iz,ix+1) + temp(iz,ix-1)) + &
                                    C2x * (temp(iz,ix+2) + temp(iz,ix-2)) + &
                                    C3x * (temp(iz,ix+3) + temp(iz,ix-3)) + &
                                    C4x * (temp(iz,ix+4) + temp(iz,ix-4)) + &
                                    C5x * (temp(iz,ix+5) + temp(iz,ix-5))

            end do
        end do
        !$OMP END PARALLEL DO

    end if

  end subroutine

end module
