program nonlinear_main

	use sep
	use pad_velocity
	use Ls_1d
	use Lr_2d
	use C5
	use weight

  	implicit none

  	integer                                   :: nz,nx,nz2,nx2,ntw,ntw2,nts,ntr,z_pad,x_pad
  	integer									  :: i,ir,is,itw,its,it2,c_1,c_2
  	integer                                   :: ns,os,ds,sz,nr,or,dr,rz,z_src,x_src
  	integer									  :: sub,shift
  	real                                      :: dts,dtw,dtw2,dtr,oz,dz,ox,dx,ots,otw
  	real									  :: otr,td,fd,courant,fmax,ratio,cr
  	real									  :: oz_src,ox_src,fz_src,fx_src,oz_in,ox_in,ox_true,oz_true
	real									  :: oz_recv,ox_recv,fz_recv,fx_recv,junk,tf,fnyqs
  	real,dimension(:,:),allocatable           :: vel,small_vel,temp1,interp_filter
  	real,dimension(:),allocatable             :: sourcew,sources,sourcew_temp,sourcew_temp_dip
  	real,dimension(:,:,:),allocatable         :: data,dataw,src_wfld
  	integer,dimension(:,:),allocatable        :: recv_coord,src_coord
  	logical									  :: dip,vel_pad,save_wfld
  	real,dimension(:,:),allocatable,target	  :: old,cur,new
  	real,dimension(:,:),pointer				  :: pold,pcur,pnew,ptemp  
	character(len=10)						  :: fmt1,fmt2,fmt3

  	call sep_init()
  	
  	! OUTPUT FORMATS
  	fmt1 = '(a,f5.2,a)'
	fmt2 = '(a)'
	fmt3 = '(a,f5.2)'

  	! IMPORTING PARAMETERS
  	call from_param("nz",nz)
  	call from_param("nx",nx)
  	call from_param("nts",nts)
  	call from_param("dts",dts)
  	call from_param("dz",dz)
  	call from_param("dx",dx)
  	call from_param("z_pad",z_pad)
  	call from_param("x_pad",x_pad)
  	call from_param("ns",ns)
  	call from_param("os",os)
  	call from_param("ds",ds)
  	call from_param("sz",sz)
  	call from_param("nr",nr)
  	call from_param("or",or)
  	call from_param("dr",dr)
  	call from_param("rz",rz)
  	call from_param("dip",dip)
  	call from_param("dipole_shift",shift,1)
  	call from_param("fmax",fmax)
  	call from_param("sub",sub)
  	call from_param("vel_pad",vel_pad,.false.)
  	call from_param("save_wfld",save_wfld,.false.) 
  	 	
  	call from_history("o1",oz_in)
  	call from_history("o2",ox_in)

 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	write(0,'(a)')"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"!!!!!!!!!!!
	write(0,'(a)')"!!!!!! FINITE-DIFFERENCE NONLINEAR FORWARD MODELING !!!!!!!"!!!!!!!!!!!
	write(0,'(a)')"!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"!!!!!!!!!!!
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	! DISPLAY WAVEFIELD RECORDING INFORMATION
  	if (dip .eqv. .true.) then
  		write(0,'(a)') "DIPOLE SOURCE"
  	else
  		write(0,'(a)') "MONOPOLE SOURCE"
  	end if
  	if (save_wfld .eqv. .true.) then
  		write(0,'(a)') "WILL WRITE SOURCE WAVEFIELD ON DISK"
  	else
  		write(0,'(a)') "WILL NOT WRITE SOURCE WAVEFIELD ON DISK"
  	end if  

 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	write(0,fmt1) "maximum frequency fmax: ",fmax," [Hz]"
	write(0,fmt1) "dz: ",dz," [km]"
	write(0,fmt1) "dx: ",dx," [km]"
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	

	os = os + x_pad 
  	or = or + x_pad 
  	sz = sz + z_pad 
  	rz = rz + z_pad 

  	ots = 0.
  	otw = 0.
  	otr = 0.

	dtw = dts / sub
	ntw = (nts-1) * sub + 1 
	dtr = dts
	ntr = nts

	fnyqs = 1 / (2*dts)
	tf = ots + (nts-1) * dts
	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	write(0,'(a,f6.3,a)') "total recording time trec: ",tf," [s]"
	write(0,'(a,i4,a)') "source number of time samples nts: ",nts," [samples]"
  	write(0,'(a,f8.5,a)') "source sampling rate dts: ",dts," [s]"
  	write(0,'(a,f6.3,a)') "Nyquist frequency: ",fnyqs," [Hz]"  	
	write(0,'(a,i6,a)') "finite-difference number of time samples ntw: ",ntw," [samples]"
  	write(0,'(a,f8.5,a)') "finite-difference sampling rate dtw: ",dtw," [s]"
	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		
  	nz2 = nz + 2 * z_pad
  	nx2 = nx + 2 * x_pad	
  
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  	write(0,'(a,i4,a)') "domain nz: ",nz," [samples]"
  	write(0,'(a,i4,a)') "domain nx: ",nx," [samples]"
  	write(0,'(a,i4,a)') "domain nz2 (after padding): ",nz2," [samples]"
  	write(0,'(a,i4,a)') "domain nx2 (after padding): ",nx2," [samples]"
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  	dtw2 = dtw * dtw
  
  	! MEMORY ALLOCATION
  	allocate(sources(nts))
  	allocate(sourcew(ntw))
  	allocate(sourcew_temp(ntw))  	
  	allocate(sourcew_temp_dip(ntw))  	  	
  	allocate(data(ntr,nr,ns))
  	allocate(dataw(ntw,nr,ns))
  	allocate(recv_coord(nr,2))
  	allocate(src_coord(ns,2))
  	allocate(vel(nz2,nx2))
  	allocate(small_vel(nz,nx))
  	allocate(old(nz2,nx2))
  	allocate(cur(nz2,nx2))
  	allocate(new(nz2,nx2))
  	allocate(temp1(nz2,nx2))
  	allocate(interp_filter(sub+1,2))
	if (save_wfld .eqv. .true.) allocate(src_wfld(nz2,nx2,nts))

  	pold => old
  	pcur => cur
  	pnew => new

	! SOURCE WAVELET
  	call sep_read(sources,"wavelet")
  	write(0,fmt2) "read source (OK)"
  	junk = maxval(abs(sources))
  	if (junk .eq. 0) write(0,fmt2) "WARNING: source is zero"	

	! VELOCITY MODEL
  	if (.not. vel_pad) then
		write(0,'(a)') "INPUT VELOCITY MODEL IS NOT PADDED"
  		call sep_read(small_vel)
  		call pad_vel(z_pad,x_pad,small_vel,vel)
  		ox = -x_pad * dx + ox_in
 		oz = -z_pad * dz + oz_in
 		oz_true = oz_in
 		ox_true = ox_in
	else
		write(0,'(a)') "INPUT VELOCITY MODEL IS PADDED"
  		call sep_read(vel)
	  	ox = ox_in
 		oz = oz_in
 		oz_true = oz + z_pad * dz
 		ox_true = ox + x_pad * dx
  	end if

 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	
	write(0,fmt1) "ox: ",ox," [km]"
	write(0,fmt1) "oz: ",oz," [km]"
	write(0,fmt1) "true ox: ",ox_true," [km]"
	write(0,fmt1) "true oz: ",oz_true," [km]"
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	 
  	if (.not. vel_pad) write(0,'(a,f5.3,a)') "max value of vel model before padding: ",maxval(small_vel)," [km/s]"
  	if (.not. vel_pad) write(0,'(a,f5.3,a)') "min value of vel model before padding: ",minval(small_vel)," [km/s]"
  	write(0,'(a,f5.3,a)') "max value of vel model after padding: ",maxval(vel)," [km/s]"
  	write(0,'(a,f5.3,a)') "min value of vel model after padding: ",minval(vel)," [km/s]"
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	 	
  	
	! FINITE-DIFFERENCE STABILITY
  	courant = maxval(vel) * dtw / min(dz,dx)
  	if (courant .ge. 0.45) then
  		write(0,fmt3) "WARNING: courant number should be < 0.45, Courant: ", courant
	else 
		write(0,'(a,f4.2,a)') "Courant: ",courant, " (OK)"
	end if

	! NUMERICAL DISPERSION
  	ratio = minval(vel) / (fmax * max(dz,dx))
  	if (ratio .ge. 3) then 
  		write(0,fmt1) "dispersion ratio: ",ratio," (OK)"
  	else
  		write(0,fmt3) "WARNING: dispersion ratio < 3: ",ratio
  	end if
  	
	! SOURCES COORDINATES
  	src_coord(:,1) = sz
  	do is=1,ns
    	src_coord(is,2) = os + ds * (is-1)
  	end do
  	oz_src = oz + (src_coord(1,1) - 1) * dz 
  	ox_src = ox + (src_coord(1,2) - 1) * dx 
  	fz_src = oz + (src_coord(ns,1) - 1) * dz 
  	fx_src = ox + (src_coord(ns,2) - 1) * dx 
  	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 	write(0,'(a,i4)') "number of shots: ",ns 	
 	write(0,'(a,f6.3,a)') "distance between two consecutive shots: ",ds*dx," [km]"
 	write(0,'(a,f6.3,a)') "depth of FIRST source: ",oz_src," [km]"
 	write(0,'(a,f6.3,a)') "offset of FIRST source: ",ox_src," [km]"	
 	write(0,'(a,f6.3,a)') "depth of LAST source: ",fz_src," [km]"
 	write(0,'(a,f6.3,a)') "offset of LAST source: ",fx_src," [km]"	  	  	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 	
	! RECEIVERS COORDINATES
  	recv_coord(:,1) = rz
  	do ir=1,nr
    	recv_coord(ir,2) = or + dr * (ir-1)
  	end do
  	oz_recv = oz + (recv_coord(1,1) - 1) * dz 
  	ox_recv = ox + (recv_coord(1,2) - 1) * dx 
  	fz_recv = oz + (recv_coord(nr,1) - 1) * dz   	
  	fx_recv = ox + (recv_coord(nr,2) - 1) * dx 
  	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 	write(0,'(a,i4.3,a)') "number of receivers: ",nr 	
 	write(0,'(a,f6.3,a)') "distance between two consecutive receivers: ",dr*dx," [km]"
 	write(0,'(a,f6.3,a)') "depth of FIRST receiver: ",oz_recv," [km]"
 	write(0,'(a,f6.3,a)')"offset of FIRST receiver: ",ox_recv," [km]"	
 	write(0,'(a,f6.3,a)') "depth of LAST receiver: ",fz_recv," [km]"
 	write(0,'(a,f6.3,a)') "offset of LAST receiver: ",fx_recv," [km]"	  	  	
 	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 		
	! PRECOMPUTE INTERPOLATION FILTER
	! WARNING: the two grids must be aligned (i.e., sub is an integer)
	do i=1,sub+1
		interp_filter(i,1) = 1.0 - 1.0 * (i-1) / sub
		interp_filter(i,2)=1.-interp_filter(i,1)
	end do
	interp_filter = interp_filter / sqrt(real(ntw)/real(nts))   		
 		
	! INTERPOLATE SOURCE
  	call Ls_1d_init(nts,ntw,ots,dts,otw,dtw)
  	call Ls_1d_op2(.false.,.false.,sources,sourcew) 

	! INITIALIZE OPERATORS
  	call Lr_2d_init(ntr,ntw,otr,dtr,otw,dtw,nr)
  	call C5_init(nz2,nx2,dtw,vel,dz,dx,z_pad,x_pad)

  	! INITIALIZE DATA
  	dataw = 0.0
  	data  = 0.0
  	src_wfld = 0.0
	
  	call system_clock(count_rate=cr)
  	call system_clock(c_1)

  	do is=1,ns ! loop over shots
  		
  		write(0,'(a,i3,a,i3)') "shot # ",is," out of ",ns
  	
  		pold = 0.0
  		pcur = 0.0 
  		pnew = 0.0

        ! updating source location 
        z_src = src_coord(is,1)
        x_src = src_coord(is,2)
  		if (save_wfld .eqv. .true.) src_wfld = 0.0        
        
 		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	 
		write(0,'(a,i4,a)') "source depth: ",z_src," [samples including absorbing boundary]"
		write(0,'(a,i4,a)') "source offset: ",x_src," [samples including absorbing boundary]"
 		!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!	         

		! scale source  
        ! no need to apply w operator since source is inside modeling domain
        ! need to scale the negative pole of the dipole with different velocity value
		sourcew_temp = 0.0
		sourcew_temp_dip = 0.0		
  		sourcew_temp = vel(z_src,x_src) * vel(z_src,x_src) * dtw2 * sourcew
		sourcew_temp_dip = vel(z_src+shift,x_src) * vel(z_src+shift,x_src) * dtw2 * sourcew
  		
		do its=1,nts-1
		
			if (modulo(its,nts/5) .eq. 0) write(0,'(a,i4,a,i4)') "its= ",its," out of ",nts
					
			do it2=2,sub+1

				itw = (its-1) * sub + it2

				! p(i) = C5 p(i-1) + (w-Id) p(i-2)
				call C5_op2(.false.,.false.,pcur,temp1)
				pnew = temp1
				call weight_op2(.false.,.false.,pold,temp1)
				pnew = pnew + temp1 - pold

				! inject source
				! p(i) = p(i) + s(i-1)
				if (dip .eqv. .true.) then
					pnew(z_src,x_src) = pnew(z_src,x_src) + sourcew_temp(itw-1)
					pnew(z_src+shift,x_src) = pnew(z_src+shift,x_src) - sourcew_temp_dip(itw-1)
				else
					pnew(z_src,x_src) = pnew(z_src,x_src) + sourcew_temp(itw-1)					
				end if
				
				! lower coarse index is its
				! higher coarse index is its+1
				! spread energy for both indices
				if (save_wfld .eqv. .true.) then
					src_wfld(:,:,its)   = src_wfld(:,:,its) + pnew * interp_filter(it2,1)
					src_wfld(:,:,its+1) = src_wfld(:,:,its+1) + pnew * interp_filter(it2,2)
				end if				
	
				! record data at dtw
				if (dip .eqv. .true.) then		
					do ir=1,nr
						dataw(itw,ir,is) = dataw(itw,ir,is) + pnew(recv_coord(ir,1),recv_coord(ir,2)) 
						dataw(itw,ir,is) = dataw(itw,ir,is) - pnew(recv_coord(ir,1)+shift,recv_coord(ir,2)) 
					end do
				else 
					do ir=1,nr
						dataw(itw,ir,is) = dataw(itw,ir,is) + pnew(recv_coord(ir,1),recv_coord(ir,2)) 
					end do				
				end if
			
				ptemp => pold
				pold => pcur 
				pcur => pnew
				pnew => ptemp	

			end do

		end do

		! interpolate data to receiver sampling for this shot		
    	call Lr_2d_op2(.true.,.false.,data(:,:,is),dataw(:,:,is))

		! finished modeling shot
  		write(0,'(a,i4,a,i4)') "finished shot # ",is," out of",ns

  	end do

  	call system_clock(c_2)
  	write(0,'(a,f12.3,a)') "time spent computing: ",(c_2-c_1)/cr," [s]"  

   	write(0,fmt2) "writing data [Pa] on disk"
  	call sep_write(data)
   	write(0,fmt2) "writing data [Pa] on disk: OK"
   	
  	call to_history("n1",nts)
  	call to_history("o1",ots)
  	call to_history("d1",dts)
  	call to_history("label1","time [s]")
  
  	call to_history("n2",nr)
  	call to_history("o2",ox_recv)
  	call to_history("d2",dx*dr)
  	call to_history("label2","receivers [km]")

  	call to_history("n3",ns)
  	call to_history("o3",ox_src)
  	call to_history("d3",dx*ds)
  	call to_history("label3","sources [km]")
  	
	if (save_wfld .eqv. .true.) then
	
  		write(0,'(a)') "saving source wavefield on disk"

		!call sep_write(src_wfld(z_pad+1:z_pad+nz,x_pad+1:x_pad+nx,:),"wfld")
		call sep_write(src_wfld,"wfld")	
		
		call to_history("n1",nz2,"wfld")  	
 		call to_history("o1",oz,"wfld")  	
 		call to_history("d1",dz,"wfld") 
  		call to_history("label1","z [km]","wfld")
 	
	 	call to_history("n2",nx2,"wfld")  	
 		call to_history("o2",ox,"wfld")  	
 		call to_history("d2",dx,"wfld") 	 		 	
  		call to_history("label2","x [km]","wfld")

 		call to_history("n3",nts,"wfld")  
 		call to_history("o3",ots,"wfld")  	
 		call to_history("d3",dts,"wfld")
  		call to_history("label3","time [s]","wfld")
  		
	end if  	
 	
 	call sep_close()

end program


