program scale_mixing

	use sep
    use fourier
	
	implicit none	
	
  	integer									  :: nz2,nx2,n3,n_ext,nbytes,index,nt
	integer									  :: ikz,ikx
	real								      :: oz,ox,dz,dx,index_okx,index_okz
	real									  :: dkz,dkx,okz,okx
	real									  :: k,kz,kx,k_taper,k_cut,k_big,k_small
  	real,dimension(:,:,:),allocatable  		  :: model
  	real,dimension(:,:),allocatable  		  :: background,reflectivity,wavelet,wavelet2	
  	real,dimension(:,:),allocatable  		  :: filter,data
  	complex,dimension(:,:),allocatable		  :: background_fft,background_filter_fft,cdata,reflectivity_fft
	integer*8								  :: srite,sreed,rc,wc	
	real									  :: sign1,sign2
	
	call sep_init(SOURCE)
	
 	call from_history("n1",nz2)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 
 	call from_history("n2",nx2)
 	call from_history("d2",dx)	
 	call from_history("o2",ox)
	
 	call from_history("n3",n3)
	
	! WAVENUMBER CUTOFF
	! every point in the kz/kx domain that has a |k| > k_cutoff should be multiplied by 0
	call from_param("k_cut",k_cut)
	call from_param("k_taper",k_taper)
	k_big = k_cut
	k_small = k_cut - k_taper

	! n_ext is the size of the extension (n_ext = nhz * nhx * nht)
 	n_ext = n3 - 1

	! index is the index that corresponds to the index of the non-extended slice of the reflectivity 
	! inside the model vector
 	index = (n_ext-1)/2 + 2 

	! ALLOCATE
 	allocate(model(nz2,nx2,n3))
 	allocate(background(nz2,nx2))
 	allocate(reflectivity(nz2,nx2))
 	allocate(background_fft(nz2,nx2))
 	allocate(background_filter_fft(nz2,nx2)) 	
 	allocate(data(nz2,nx2)) 
 	allocate(cdata(nz2,nx2))  		
 	allocate(reflectivity_fft(nz2,nx2))	
 	allocate(filter(nz2,nx2))

	! FFT PARAMETERS
	sign1 = 1.0 
	sign2 = 1.0 	
	
  	dkz = 1./((nz2-1)*dz)
  	okz = -1./(2*dz)
  	dkx = 1./((nx2-1)*dx)
  	okx = -1./(2*dx)  		
	
	! READ MODEL
 	nbytes = 4
 	nbytes = nbytes * nx2 * nz2 * n3
 	rc = sreed("in",model,nbytes)

	! INITIALIZE FILTER TO ZERO
	filter = 0.0

	! EXTRACT BACKGROUND
 	background_fft = model(:,:,1)

	! APPLY FFT TO BACKGROUND
 	call ft_init(nz2,nx2)
 	call ft1axis(.false.,sign1,background_fft)
 	!call ft2axis(.false.,sign2,background_fft)	

	! WRITE FFT BEFORE FILTERING
	call sep_write(real(background_fft))
	
 	call to_history("n1",nz2)
 	call to_history("d1",dkz)	
 	call to_history("o1",okz)
 	call to_history("label1","kz [1/km]")
 	 
 	call to_history("n2",nx2)
 	call to_history("d2",dkx)	
 	call to_history("o2",okx)
 	call to_history("label2","kx [1/km]")

	! COMPUTE LOW-PASS FILTER
	index_okx = (nx2-1)/2 + 1
	index_okz = (nz2-1)/2 + 1
	
!  	do ikx=1,nx2
!  
!  		kx = (ikx - index_okx) * dkx
!  		do ikz=1,nz2
!  			kz = (ikz - index_okz) * dkz			
!  			k = sqrt(kz*kz+kx*kx) 
!  
!  			if (k .lt. k_small) then  
!  				filter(ikz,ikx) = 1.0 
!  			else if ((k .ge.  k_cut - k_taper) .and. (k .lt.  k_cut)) then
!  				filter(ikz,ikx) = cos(3.14159265359/2.0 * (k-k_small)/(k_big-k_small))**2
!  			end if
!  				
!  		end do	
!  
!  	end do
 	
 	do ikz=1,nz2
 		kz = (ikz - index_okz) * dkz			
 			k =abs(kx) 
  			if (k .lt. k_small) then  
 				filter(ikz,ikx) = 1.0 
 			else if ((k .ge.  k_cut - k_taper) .and. (k .lt.  k_cut)) then
 				filter(ikz,ikx) = cos(3.14159265359/2.0 * (k-k_small)/(k_big-k_small))**2
 			end if
 				
 		end do	
 
 	end do 	
	
	! APPLY FFT FILTER
	background_filter_fft = background_fft * filter
	
	! WRITE FFT BEFORE FILTERING
	call sep_write(real(background_filter_fft),"fft_filter")
	
 	call to_history("n1",nz2,"fft_filter")
 	call to_history("d1",dkz,"fft_filter")
 	call to_history("o1",okz,"fft_filter")
 	call to_history("label1","kz [1/km]","fft_filter")
 	 
 	call to_history("n2",nx2,"fft_filter")
 	call to_history("d2",dkx,"fft_filter")
 	call to_history("o2",okx,"fft_filter")
 	call to_history("label2","kx [1/km]","fft_filter")	
 
 	cdata = background_filter_fft
 
	! APPLY IFFT TO FILTERED BACKGROUND FFT
 	call ft1axis(.true.,sign1,cdata)
 	call ft2axis(.true.,sign2,cdata)
	data = real(cdata)
	
	! WRITE FILTERED MODEL
	call sep_write(data,"time_domain")
	
 	call to_history("n1",nz2,"time_domain")
 	call to_history("d1",dz,"time_domain")
 	call to_history("o1",oz,"time_domain")
 	call to_history("label1","z [km]","time_domain")
 	 
 	call to_history("n2",nx2,"time_domain")
 	call to_history("d2",dx,"time_domain")
 	call to_history("o2",ox,"time_domain")
 	call to_history("label2","x [km]","time_domain")
	 	
 	call sep_close()

end program 