program scale_mixing_1d

	use sep
    use fourier
	
	implicit none	
	
  	integer									  :: nz2,nx2,n3,n_ext,nbytes,index,nt,n_left
	integer									  :: ikz,ikx
	real								      :: oz,ox,dz,dx,index_okx,index_okz
	real									  :: dkz,dkx,okz,okx
	real									  :: k,kz,kx,k_taper,k_cut,k_big,k_small
  	real,dimension(:,:),allocatable  		  :: model,data,filter
  	complex,dimension(:,:),allocatable		  :: model_fft,model_fft_filter,cdata
	real									  :: sign1,sign2
	
	call sep_init(SOURCE)
	
 	call from_history("n1",nz2)
 	call from_history("d1",dz)	
 	call from_history("o1",oz)
 	
 	! FFT PARAMETERS
	sign1 = 1.0 
  	dkz = 1./((nz2-1)*dz)
	n_left = (nz2-(nz2+1)/2)
	okz = -1.0 * n_left * dkz 	
 
	! WAVENUMBER CUTOFF
	! every point in the kz/kx domain that has a |k| > k_cutoff should be multiplied by 0
	call from_param("k_cut",k_cut)
	call from_param("k_taper",k_taper)

	! CHECK
	if(k_taper .ge. k_cut) then 
		k_taper = k_cut - dkz
		write(0,*)"WARNING k_taper greater than k_cut!!!"
		write(0,*)"Changing k_taper to:", k_taper
	end if

	! CUTOFFS
	k_big = k_cut
	k_small = k_cut - k_taper

	! ALLOCATE
 	allocate(model(nz2,1))
	allocate(model_fft(nz2,1))
	allocate(model_fft_filter(nz2,1))
	allocate(data(nz2,1))
	allocate(cdata(nz2,1))
	allocate(filter(nz2,1))
		
	! READ MODEL
	call sep_read(model)

	! INITIALIZE FILTER TO ZERO
	filter = 0.0

	! APPLY FFT TO MODEL
	model_fft = model
 	call ft_init(nz2,1)
 	call ft1axis(.false.,sign1,model_fft)

	! WRITE FFT BEFORE FILTERING
	call sep_write(cabs(model_fft))
	
 	call to_history("n1",nz2)
 	call to_history("d1",dkz)	
 	call to_history("o1",okz)
 	call to_history("label1","kz [1/km]")
 	 
	! COMPUTE LOW-PASS FILTER
	index_okz = (nz2-1)/2 + 1
	do ikz=1,nz2
 		kz = (ikz - index_okz) * dkz			
 			k = abs(kz) 
  			if (k .lt. k_small) then  
 				filter(ikz,1) = 1.0 
 			else if ((k .ge.  k_cut - k_taper) .and. (k .lt.  k_cut)) then
 				filter(ikz,1) = cos(3.14159265359/2.0 * (k-k_small)/(k_big-k_small))**2
 			end if
  	end do 	
	
	! APPLY FFT FILTER
	model_fft_filter = model_fft * filter
	
	! WRITE FFT BEFORE FILTERING
	call sep_write(cabs(model_fft_filter),"out_fft")
	
 	call to_history("n1",nz2,"out_fft")
 	call to_history("d1",dkz,"out_fft")
 	call to_history("o1",okz,"out_fft")
 	call to_history("label1","kz [1/km]","out_fft")
 	 
 	cdata = model_fft_filter
 
	! APPLY IFFT
 	call ft1axis(.true.,sign1,cdata)
	data = real(cdata)
	
	! WRITE FILTERED MODEL
	call sep_write(data,"out_time")
	
 	call to_history("n1",nz2,"out_time")
 	call to_history("d1",dz,"out_time")
 	call to_history("o1",oz,"out_time")
 	call to_history("label1","z [km]","out_time")
 	 
 	! COMPLEMENTARY 
  	data = model - data
  	call sep_write(data,"out_time_comp")
  	 
  	call to_history("n1",nz2,"out_time_comp")
  	call to_history("d1",dz,"out_time_comp")
  	call to_history("o1",oz,"out_time_comp")
  	call to_history("label1","z [km]","out_time_comp") 
 	 
 	call sep_close()

end program 