module weight
 
	implicit none

  	integer,private                           :: nz,nx,z_pad,x_pad,nzs,nxs
  	real,private                              :: dz,dx,dt
  	real,dimension(:,:),allocatable,private   :: vel,w_

  	contains

  	subroutine weight_init(nz_in,nx_in,dt_in,vel_in,dz_in,dx_in,z_pad_in,x_pad_in)
    	integer                  :: nz_in,nx_in,z_pad_in,x_pad_in,i,j
    	real                     :: dt_in,dz_in,dx_in
    	real,dimension(:,:)      :: vel_in

    	nz = nz_in
	   	nx = nx_in 
	   	dz = dz_in
    	dx = dx_in
    	dt = dt_in
    	z_pad = z_pad_in
    	x_pad = x_pad_in

		nxs = nx - 2 * x_pad
		nzs = nz - 2 * z_pad

    	if (.not. allocated(vel)) allocate(vel(nz,nx))
    	if (.not. allocated(w_)) allocate(w_(nz,nx))

    	vel = vel_in
    	w_  = 0.0

    	do i=1,z_pad
        	w_(i,:)      = w_(i,:)      + float(z_pad-i+1)/float(z_pad) * 0.15 * vel(i,:)      * dt/dz
        	w_(nz-i+1,:) = w_(nz-i+1,:) + float(z_pad-i+1)/float(z_pad) * 0.15 * vel(nz-i+1,:) * dt/dz
    	end do
    	do i=1,x_pad
        	w_(:,i)      = w_(:,i)      + float(x_pad-i+1)/float(x_pad) * 0.15 * vel(:,i)      * dt/dx
        	w_(:,nx-i+1) = w_(:,nx-i+1) + float(x_pad-i+1)/float(x_pad) * 0.15 * vel(:,nx-i+1) * dt/dx
    	end do


	end subroutine

  	function weight_op(adj,add,model,data) result(stat)
    	logical,intent(in)     :: adj,add
    	real,dimension(:)      :: model,data
    	integer                :: stat
    	call weight_op2(adj,add,model,data)
    	stat=0
  	end function

  	subroutine weight_op2(adj,add,model,data)
    	logical,intent(in)               :: adj,add
    	real,dimension(nz,nx)            :: model,data

    	if (adj) then

        	if(add .eqv. .false.) model = 0.

			! z-weight
        	model(1:z_pad,:) = model(1:z_pad,:) + data(1:z_pad,:) * w_(1:z_pad,:)
        	model(nz-z_pad+1:nz,:) = model(nz-z_pad+1:nz,:) + data(nz-z_pad+1:nz,:) * w_(nz-z_pad+1:nz,:)
		
			! x-weight		
        	model(z_pad+1:z_pad+nzs,1:x_pad) = model(z_pad+1:z_pad+nzs,1:x_pad) + data(z_pad+1:z_pad+nzs,1:x_pad) * w_(z_pad+1:z_pad+nzs,1:x_pad)
        	model(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) = model(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) + data(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) * w_(z_pad+1:z_pad+nzs,nx-x_pad+1:nx)

    	else

    		if(add .eqv. .false.) data = 0.

			! z-weight
        	data(1:z_pad,:) = data(1:z_pad,:) + model(1:z_pad,:) * w_(1:z_pad,:)
        	data(nz-z_pad+1:nz,:) = data(nz-z_pad+1:nz,:) + model(nz-z_pad+1:nz,:) * w_(nz-z_pad+1:nz,:)
		
			! x-weight		
        	data(z_pad+1:z_pad+nzs,1:x_pad) = data(z_pad+1:z_pad+nzs,1:x_pad) + model(z_pad+1:z_pad+nzs,1:x_pad) * w_(z_pad+1:z_pad+nzs,1:x_pad)
        	data(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) = data(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) + model(z_pad+1:z_pad+nzs,nx-x_pad+1:nx) * w_(z_pad+1:z_pad+nzs,nx-x_pad+1:nx)

    	end if
	end subroutine

  	subroutine weight_close()
		deallocate(w_)
    	deallocate(vel)
  	end subroutine

end module
