##########################################################################################
###################################### WAVELET ###########################################
##########################################################################################
# 45Hz wavelet (full bandwidth)
Wlt/big.sed.w.45.H: exec
	${B}/wavelet.x par=Par/big.sed.fwd.p fd=15 td=1.8 > temp1.H
	Scale < temp1.H > $@

# 5Hz wavelet 
Wlt/big.sed.w.5.H: Wlt/big.sed.w.45.H exec
	Bandpass fhi=5 < Wlt/big.sed.w.45.H | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 > $@	

##########################################################################################
###################################### FWI MODELS ########################################
##########################################################################################	

Vel/fwi.start.H: exec
	Pad beg1=500 end1=500 beg2=300 end2=300 extend=1 < Vel/fwi.sed.vel.H > junk1.H
	Smooth rect1=20 rect2=20 repeat=5 < junk1.H > junk2.H
	Window3d n1=500 f1=500 n2=2000 f2=500 < junk2.H > junk3.H
	Window3d n1=450 f1=50 < junk3.H > junk4.H
	echo "o1=0.0" >> junk4.H
	Window3d f1=0 n1=50 < Vel/fwi.sed.vel.H > water_layer.H
	Cat axis=1 water_layer.H junk4.H > $@
	Rm junk*.H

Vel/fwi.start.p.H: Vel/fwi.start.H  exec
	Pad beg1=100 end1=100 beg2=100 end2=100 extend=1 < Vel/fwi.start.H > $@


##########################################################################################
###################################### DATA ##############################################
##########################################################################################	
# 45Hz data (full bandwidth)
Dat/big.sed.dat.45.H: batch_task_executor_nonlinear.py Wlt/big.sed.w.45.H exec
	$< pbs_template=$(PBS_TMPL) YANG_BIN=/data/sep/zyang03/bin os=1 nshots=200 nshotjob=2 ds=10 vel=../Vel/fwi.sed.vel.H par=../Par/big.sed.fwd.p queues=$(SEPQ),default queues_cap=30,30 path_out=wrk prefix=datbigsed total_jobs_cap=24,200
	Cp datbigsed-final.H Dat/big.sed.dat.45.H

# 45Hz data (full bandwidth)
Dat/big.sed.dat.45.small.H: Dat/big.sed.dat.45.H exec
	Window3d n3=2 < Dat/big.sed.dat.45.H > $@

# Bandpass the data at 5Hz (apply the same filter as for the wavelet)
Dat/big.sed.dat.5.H: Dat/big.sed.dat.45.H exec
	Bandpass fhi=5 < Dat/big.sed.dat.45.H | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 | Bandpass fhi=5 > $@


##########################################################################################
############################ APPROXIMATE HESSIAN COMPUTATION #############################
##########################################################################################	
# generate a single spike model with only three points to see how wide the spread is after applying (B*)B
Vel/big.sed.single.spike.H: exec
	Spike n1=500 n2=2000 nsp=1 k1=250 k2=1000 > junk1.H
	echo "o1=0.0 d1=0.01 o2=0.0 d2=0.01" >> junk1.H
	Pad beg1=100 end1=100 beg2=100 end2=100 extend=0 < junk1.H > $@

# fwd Born
big.sed.hessian.dat.single.5.H: batch_task_executor_born_fwd.py Vel/fwi.start.p.H Vel/big.sed.single.spike.H Wlt/big.sed.w.5.H exec
	batch_task_executor_born_fwd.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 model=../Vel/big.sed.single.spike.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=30,30 path_out=wrk prefix=bigsingleseddathessian total_jobs_cap=24,200
	Cp bigsingleseddathessian-final.H $@		

# adj Born	
big.sed.hessian.model.single.5.H: batch_task_executor_born_adj.py big.sed.hessian.dat.single.5.H Wlt/big.sed.w.5.H exec
	batch_task_executor_born_adj.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 data=../big.sed.hessian.dat.single.5.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=30,30 path_out=wrk prefix=bigsinglesedmodhessian total_jobs_cap=24,200
	Cp bigsinglesedmodhessian-final.H $@

# generate small spike model with only three points to see how wide the spread is after applying (B*)B
Vel/big.sed.spike.H: exec
	Spike n1=500 n2=2000 nsp=3 k1=150,400,200 k2=400,1000,1600 > junk1.H
	echo "o1=0.0 d1=0.01 o2=0.0 d2=0.01" >> junk1.H
	Pad beg1=100 end1=100 beg2=100 end2=100 extend=0 < junk1.H > $@
	
# fwd Born
big.sed.hessian.dat.temp.5.H: batch_task_executor_born_fwd.py Vel/big.sed.spike.H Vel/fwi.start.p.H Wlt/big.sed.w.5.H exec
	batch_task_executor_born_fwd.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 model=../Vel/big.sed.spike.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=30,30 path_out=wrk prefix=bigseddathessian total_jobs_cap=24,200
	Cp bigseddathessian-final.H $@		
		
# adj Born	
big.sed.hessian.model.temp.5.H: batch_task_executor_born_adj.py big.sed.hessian.dat.temp.5.H Wlt/big.sed.w.5.H exec
	batch_task_executor_born_adj.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 data=../big.sed.hessian.dat.temp.5.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=30,30 path_out=wrk prefix=bigsedmodhessian total_jobs_cap=24,200
	Cp bigsedmodhessian-final.H $@		

# generate a denser spike model
Vel/big.sed.spike.dense.H: exec
	Spike n1=500 n2=2000 nsp=9 k1=100,100,100,280,280,280,460,460,460 k2=200,1000,1800,200,1000,1800,200,1000,1800 > junk1.H
	echo "o1=0.0 d1=0.01 o2=0.0 d2=0.01" >> junk1.H
	Pad beg1=100 end1=100 beg2=100 end2=100 extend=0 < junk1.H > $@

# fwd Born	
big.sed.hessian.dat.5.H: batch_task_executor_born_fwd.py Vel/big.sed.spike.dense.H Vel/fwi.start.p.H Wlt/big.sed.w.5.H exec
	batch_task_executor_born_fwd.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 model=../Vel/big.sed.spike.dense.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=0,30 path_out=wrk prefix=bigseddathessian total_jobs_cap=0,200
	Cp bigseddathessian-final.H $@			
	

# adj Born	
big.sed.hessian.model.5.H: batch_task_executor_born_adj.py big.sed.hessian.dat.5.H exec
	batch_task_executor_born_adj.py pbs_template=/home/gbarnier/script/pbs_tmpl_mazama.sh os=1 nshots=200 nshotjob=2 ds=10 data=../big.sed.hessian.dat.5.H vel=../Vel/fwi.start.p.H par=../Par/big.sed.hessian.fwd.p queues=sep,default queues_cap=0,30 path_out=wrk prefix=bigsedmodhessian total_jobs_cap=0,200
	Cp bigsedmodhessian-final.H $@	


# Interpolate and pad
big.sed.hessian.weight.H: big.sed.hessian.model.5.H Vel/big.sed.spike.dense.H exec
	Window3d n1=500 f1=100 n2=2000 f2=100 < big.sed.hessian.model.5.H > junk1.H
	Window3d n1=500 f1=100 n2=2000 f2=100 < Vel/big.sed.spike.dense.H > pippo1.H	
	Window3d f1=99 j1=180 f2=199 j2=800 < junk1.H > junk2.H
	Window3d f1=99 j1=180 f2=199 j2=800 < pippo1.H > pippo2.H
	Interp o1out=0.0 d1out=0.01 o2out=0.0 d2out=0.01 n1out=500 n2out=2000 type=1 < junk2.H > junk3.H
	Math file1=junk3.H exp='1/(file1+1e-16)' | Pad beg1=100 end1=100 beg2=100 end2=100 extend=1 > $@	

# Scale the weighting operator
big.sed.hessian.weight.scale.H: big.sed.hessian.weight.H exec
	Scale < big.sed.hessian.weight.H > $@
	Window3d n2=1 f2=1000 < big.sed.hessian.weight.scale.H | Graph | Tube &
	
big.sed.hessian.weight.scale.water.mask.H: big.sed.hessian.weight.scale.H exec
	Window3d n1=500 f1=100 n2=2000 f2=100 < big.sed.hessian.weight.scale.H > junk1.H
	Window3d f1=50 n1=450 < junk1.H > junk2.H
	Smooth rect1=20 rect2=20 < junk2.H > junk2.smooth.H
	echo "o1=0.0 o2=0.0" >> junk2.H
	echo "o1=0.0 o2=0.0" >> junk2.smooth.H
	Vel n1=50 n2=2000 n3=1 d1=0.010 d2=0.010 d3=0.010 o1=0 o2=0 o3=0 vc=0.0 > water_layer.H
	Cat axis=1 water_layer.H junk2.H | Pad beg1=100 end1=100 beg2=100 end2=100 extend=1 > $@
	Cat axis=1 water_layer.H junk2.smooth.H | Pad beg1=100 end1=100 beg2=100 end2=100 extend=1 > big.sed.hessian.weight.scale.water.mask.smooth.H 
	
big.sed.hessian.ones.H: exec
	Vel n1=50 n2=2000 n3=1 d1=0.010 d2=0.010 d3=0.010 o1=0.0 o2=0.0 o3=0.0 vc=0.0 > water_layer.H		
	Vel n1=450 n2=2000 n3=1 d1=0.010 d2=0.010 d3=0.010 o1=0.0 o2=0.0 o3=0.0 vc=1.0 > ones.H 
	Cat axis=1 water_layer.H ones.H | Pad beg1=100 end1=100 beg2=100 end2=100 extend=1 > $@		
		
##########################################################################################
###################################### INVERSIONS ########################################
##########################################################################################
big.sed.hessian.inversion.5:  Vel/fwi.start.p.H Wlt/big.sed.w.5.H big.sed.hessian.weight.scale.water.mask.H Dat/big.sed.dat.5.H exec
	./python_solver/generic_non_linear_prob.py fwd_nl_cmd_file=Par/big.sed.hessian.nonlinear.p fwd_cmd_file=Par/big.sed.hessian.born.fwd.p adj_cmd_file=Par/big.sed.hessian.born.adj.p data=Dat/big.sed.dat.5.H init_model=Vel/fwi.start.p.H inv_model=big.sed.hessian.5.H suffix=big.sed.hessian.5 iteration_movies=obj,model,gradient,residual niter=40 dotprod=0 log_file=fwi.taper.inversion.log wrk_dir=BigSedHessian5

big.sed.hessian.inversion.no.water.5:  Vel/fwi.start.p.H Wlt/big.sed.w.5.H Dat/big.sed.dat.5.H exec
	./python_solver/generic_non_linear_prob.py fwd_nl_cmd_file=Par/big.sed.hessian.nonlinear.p fwd_cmd_file=Par/big.sed.hessian.born.fwd.p adj_cmd_file=Par/big.sed.hessian.born.adj.p data=Dat/big.sed.dat.5.H init_model=Vel/fwi.start.p.H inv_model=big.sed.hessian.5.no.water.H suffix=big.sed.hessian.5.no.water iteration_movies=obj,model,gradient,residual niter=40 dotprod=0 log_file=fwi.taper.inversion.log wrk_dir=BigSedHessian5Nw









	
